/*
 * Copyright (C) 2008-2012 The QXmpp developers
 *
 * Author:
 *  Manjeet Dahiya
 *
 * Source:
 *  http://code.google.com/p/qxmpp
 *
 * This file is a part of QXmpp library.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 */


#include <QBuffer>
#include <QXmlStreamWriter>

#include "QXmppVCardIq.h"
#include "QXmppUtils.h"
#include "QXmppConstants.h"

QXmppVCardPhone::QXmppVCardPhone() :
    isHome(false),
    isWork(false),
    isVoice(false),
    isFax(false),
    isPager(false),
    isMsg(false),
    isCell(false),
    isVideo(false),
    isBBS(false),
    isModem(false),
    isISDN(false),
    isPCS(false),
    isPref(false)
{
}

QXmppVCardEmail::QXmppVCardEmail() :
    isHome(false),
    isWork(false),
    isInternet(false),
    isPref(false),
    isX400(false)
{
}

QXmppVCardAddress::QXmppVCardAddress() :
    isHome(false),
    isWork(false),
    isPostal(false),
    isPref(false)
{
}

bool QXmppVCardAddress::isEmpty() const
{
    return street.isEmpty() && locality.isEmpty() &&
            region.isEmpty() && pCode.isEmpty() &&
            country.isEmpty() && extAdd.isEmpty();
}

static QString getImageType(const QByteArray &contents)
{
    if (contents.startsWith("\x89PNG\x0d\x0a\x1a\x0a"))
        return "image/png";
    else if (contents.startsWith("\x8aMNG"))
        return "video/x-mng";
    else if (contents.startsWith("GIF8"))
        return "image/gif";
    else if (contents.startsWith("BM"))
        return "image/bmp";
    else if (contents.contains("/* XPM */"))
        return "image/x-xpm";
    else if (contents.contains("<?xml") && contents.contains("<svg"))
        return "image/svg+xml";
    else if (contents.startsWith("\xFF\xD8\xFF\xE0"))
        return "image/jpeg";
    return "image/unknown";
}

/// Constructs a QXmppVCardIq for the specified recipient.
///
/// \param jid

QXmppVCardIq::QXmppVCardIq(const QString& jid) : QXmppIq(QXmppIq::Get)
{
    // for self jid should be empty
    setTo(jid);
}

/// Returns the date of birth of the individual associated with the vCard.
///

QDate QXmppVCardIq::birthday() const
{
    return m_birthday;
}

/// Sets the date of birth of the individual associated with the vCard.
///
/// \param birthday

void QXmppVCardIq::setBirthday(const QDate &birthday)
{
    m_birthday = birthday;
}

/// Returns the email address.
///

QString QXmppVCardIq::email() const
{
    return m_email;
}

/// Sets the email address.
///
/// \param email

void QXmppVCardIq::setEmail(const QString &email)
{
    m_email = email;
}

/// Returns the first name.
///

QString QXmppVCardIq::firstName() const
{
    return m_firstName;
}

/// Sets the first name.
///
/// \param firstName

void QXmppVCardIq::setFirstName(const QString &firstName)
{
    m_firstName = firstName;
}

/// Returns the full name.
///

QString QXmppVCardIq::fullName() const
{
    return m_fullName;
}

/// Sets the full name.
///
/// \param fullName

void QXmppVCardIq::setFullName(const QString &fullName)
{
    m_fullName = fullName;
}

/// Returns the last name.
///

QString QXmppVCardIq::lastName() const
{
    return m_lastName;
}

/// Sets the last name.
///
/// \param lastName

void QXmppVCardIq::setLastName(const QString &lastName)
{
    m_lastName = lastName;
}

/// Returns the middle name.
///

QString QXmppVCardIq::middleName() const
{
    return m_middleName;
}

/// Sets the middle name.
///
/// \param middleName

void QXmppVCardIq::setMiddleName(const QString &middleName)
{
    m_middleName = middleName;
}

/// Returns the nickname.
///

QString QXmppVCardIq::nickName() const
{
    return m_nickName;
}

/// Sets the nickname.
///
/// \param nickName

void QXmppVCardIq::setNickName(const QString &nickName)
{
    m_nickName = nickName;
}

/// Returns the URL associated with the vCard. It can represent the user's
/// homepage or a location at which you can find real-time information about
/// the vCard.

QString QXmppVCardIq::url() const
{
    return m_url;
}

/// Sets the URL associated with the vCard. It can represent the user's
/// homepage or a location at which you can find real-time information about
/// the vCard.
///
/// \param url

void QXmppVCardIq::setUrl(const QString& url)
{
    m_url = url;
}

/// Returns the name of the organization the user works in.

QString QXmppVCardIq::orgName() const
{
    return m_orgName;
}

/// Sets the name of the organization the user works in.

void QXmppVCardIq::setOrgName(const QString &orgName)
{
    m_orgName = orgName;
}

/// Returns the unit of the organization the user works in.

QString QXmppVCardIq::orgUnit() const
{
    return m_orgUnit;
}

/// Sets the unit of the ogranization the user works in.

void QXmppVCardIq::setOrgUnit(const QString &orgUnit)
{
    m_orgUnit = orgUnit;
}

/// Returns the title of the user's job.

QString QXmppVCardIq::title() const
{
    return m_title;
}

/// Sets the title of the user's job.

void QXmppVCardIq::setTitle(const QString &title)
{
    m_title = title;
}

/// Returns the role of the user in his organization.

QString QXmppVCardIq::role() const
{
    return m_role;
}

/// Sets the role of the user in his organization.

void QXmppVCardIq::setRole(const QString &role)
{
    m_role = role;
}

QString QXmppVCardIq::desc() const
{
    return m_desc;
}

void QXmppVCardIq::setDesc(const QString &desc)
{
    m_desc = desc;
}

QXmppVCardPhoneList QXmppVCardIq::phones() const
{
    return m_phones;
}

void QXmppVCardIq::setPhones(const QXmppVCardPhoneList &phones)
{
    m_phones = phones;
}

QXmppVCardEmailList QXmppVCardIq::emails() const
{
    return m_emails;
}

void QXmppVCardIq::setEmails(const QXmppVCardEmailList &emails)
{
    m_emails = emails;
}

QXmppVCardAddressList QXmppVCardIq::addresses() const
{
    return m_addresses;
}

void QXmppVCardIq::setAddresses(const QXmppVCardAddressList &addresses)
{
    m_addresses = addresses;
}

/// Returns the photo's binary contents.
///
/// If you want to use the photo as a QImage you can use:
///
/// \code
/// QBuffer buffer;
/// buffer.setData(myCard.photo());
/// buffer.open(QIODevice::ReadOnly);
/// QImageReader imageReader(&buffer);
/// QImage myImage = imageReader.read();
/// \endcode

QByteArray QXmppVCardIq::photo() const
{
    return m_photo;
}

/// Sets the photo's binary contents.

void QXmppVCardIq::setPhoto(const QByteArray& photo)
{
    m_photo = photo;
}

/// Returns the photo's MIME type.

QString QXmppVCardIq::photoType() const
{
    return m_photoType;
}

/// Sets the photo's MIME type.

void QXmppVCardIq::setPhotoType(const QString& photoType)
{
    m_photoType = photoType;
}

/// \cond
bool QXmppVCardIq::isVCard(const QDomElement &nodeRecv)
{
    return nodeRecv.firstChildElement("vCard").namespaceURI() == ns_vcard;
}

void QXmppVCardIq::parseElementFromChild(const QDomElement& nodeRecv)
{
    // vCard
    QDomElement cardElement = nodeRecv.firstChildElement("vCard");
    m_birthday = QDate::fromString(cardElement.firstChildElement("BDAY").text(), "yyyy-MM-dd");
    QDomElement emailElement = cardElement.firstChildElement("EMAIL");
    m_email = emailElement.firstChildElement("USERID").text();
    m_fullName = cardElement.firstChildElement("FN").text();
    m_nickName = cardElement.firstChildElement("NICKNAME").text();
    QDomElement nameElement = cardElement.firstChildElement("N");
    m_firstName = nameElement.firstChildElement("GIVEN").text();
    m_lastName = nameElement.firstChildElement("FAMILY").text();
    m_middleName = nameElement.firstChildElement("MIDDLE").text();

    m_url = cardElement.firstChildElement("URL").text();

    m_desc = cardElement.firstChildElement("DESC").text();

    QDomElement photoElement = cardElement.firstChildElement("PHOTO");
    QByteArray base64data = photoElement.
                            firstChildElement("BINVAL").text().toAscii();
    m_photo = QByteArray::fromBase64(base64data);
    m_photoType = photoElement.firstChildElement("TYPE").text();

    // organization stuff
    QDomElement orgElement = cardElement.firstChildElement("ORG");
    m_orgName = orgElement.firstChildElement("ORGNAME").text();
    m_orgUnit = orgElement.firstChildElement("ORGUNIT").text();
    m_title = cardElement.firstChildElement("TITLE").text();
    m_role = cardElement.firstChildElement("ROLE").text();

    QDomElement telElement = cardElement.firstChildElement("TEL");
    while (!telElement.isNull())
    {
        QXmppVCardPhone phone;
        phone.isHome = !telElement.firstChildElement("HOME").isNull();
        phone.isWork = !telElement.firstChildElement("WORK").isNull();
        phone.isVoice = !telElement.firstChildElement("VOICE").isNull();
        phone.isFax = !telElement.firstChildElement("FAX").isNull();
        phone.isPager = !telElement.firstChildElement("PAGER").isNull();
        phone.isMsg = !telElement.firstChildElement("MSG").isNull();
        phone.isCell = !telElement.firstChildElement("CELL").isNull();
        phone.isVideo = !telElement.firstChildElement("VIDEO").isNull();
        phone.isBBS = !telElement.firstChildElement("BBS").isNull();
        phone.isModem = !telElement.firstChildElement("MODEM").isNull();
        phone.isISDN = !telElement.firstChildElement("ISDN").isNull();
        phone.isPCS = !telElement.firstChildElement("PCS").isNull();
        phone.isPref = !telElement.firstChildElement("PREF").isNull();
        phone.number = telElement.firstChildElement("NUMBER").text();

        m_phones << phone;

        telElement = telElement.nextSiblingElement("TEL");
    }

    emailElement = cardElement.firstChildElement("EMAIL");
    while (!emailElement.isNull())
    {
        QXmppVCardEmail email;
        email.isHome = !emailElement.firstChildElement("HOME").isNull();
        email.isWork = !emailElement.firstChildElement("WORK").isNull();
        email.isInternet = !emailElement.firstChildElement("INTERNET").isNull();
        email.isPref = !emailElement.firstChildElement("PREF").isNull();
        email.isX400 = !emailElement.firstChildElement("X400").isNull();
        email.address = emailElement.firstChildElement("USERID").text();

        m_emails << email;

        emailElement = emailElement.nextSiblingElement("EMAIL");
    }

    QDomElement addrElement = cardElement.firstChildElement("ADR");
    while (!addrElement.isNull())
    {
        QXmppVCardAddress address;
        address.isHome = !addrElement.firstChildElement("HOME").isNull();
        address.isWork = !addrElement.firstChildElement("WORK").isNull();
        address.isPostal = !addrElement.firstChildElement("POSTAL").isNull();
        address.isPref = !addrElement.firstChildElement("PREF").isNull();

        address.country = addrElement.firstChildElement("CTRY").text();
        address.extAdd = addrElement.firstChildElement("EXTADD").text();
        address.locality = addrElement.firstChildElement("LOCALITY").text();
        address.pCode = addrElement.firstChildElement("PCODE").text();
        address.region = addrElement.firstChildElement("REGION").text();
        address.street = addrElement.firstChildElement("STREET").text();

        m_addresses << address;

        addrElement = addrElement.nextSiblingElement("ADR");
    }
}

void QXmppVCardIq::toXmlElementFromChild(QXmlStreamWriter *writer) const
{
    writer->writeStartElement("vCard");
    writer->writeAttribute("xmlns", ns_vcard);
    if (m_birthday.isValid())
        helperToXmlAddTextElement(writer, "BDAY", m_birthday.toString("yyyy-MM-dd"));
    if (!m_fullName.isEmpty())
        helperToXmlAddTextElement(writer, "FN", m_fullName);
    if(!m_nickName.isEmpty())
        helperToXmlAddTextElement(writer, "NICKNAME", m_nickName);
    if (!m_firstName.isEmpty() ||
        !m_lastName.isEmpty() ||
        !m_middleName.isEmpty())
    {
        writer->writeStartElement("N");
        if (!m_firstName.isEmpty())
            helperToXmlAddTextElement(writer, "GIVEN", m_firstName);
        if (!m_lastName.isEmpty())
            helperToXmlAddTextElement(writer, "FAMILY", m_lastName);
        if (!m_middleName.isEmpty())
            helperToXmlAddTextElement(writer, "MIDDLE", m_middleName);
        writer->writeEndElement();
    }
    if (!m_url.isEmpty())
        helperToXmlAddTextElement(writer, "URL", m_url);

    if(!photo().isEmpty())
    {
        writer->writeStartElement("PHOTO");
        QString photoType = m_photoType;
        if (photoType.isEmpty())
            photoType = getImageType(m_photo);
        helperToXmlAddTextElement(writer, "TYPE", photoType);
        helperToXmlAddTextElement(writer, "BINVAL", m_photo.toBase64());
        writer->writeEndElement();
    }

    if (!m_orgName.isEmpty() || !m_orgUnit.isEmpty())
    {
        writer->writeStartElement("ORG");
        helperToXmlAddTextElement(writer, "ORGNAME", m_orgName);
        helperToXmlAddTextElement(writer, "ORGUNIT", m_orgUnit);
        writer->writeEndElement();
    }

    if (!m_title.isEmpty())
        helperToXmlAddTextElement(writer, "TITLE", m_title);
    if (!m_role.isEmpty())
        helperToXmlAddTextElement(writer, "ROLE", m_role);
    if (!m_desc.isEmpty())
        helperToXmlAddTextElement(writer, "DESC", m_desc);

    foreach (const QXmppVCardPhone &phone, m_phones)
    {
        writer->writeStartElement("TEL");

        if (phone.isHome)
            writer->writeEmptyElement("HOME");
        if (phone.isWork)
            writer->writeEmptyElement("WORK");
        if (phone.isVoice)
            writer->writeEmptyElement("VOICE");
        if (phone.isFax)
            writer->writeEmptyElement("FAX");
        if (phone.isPager)
            writer->writeEmptyElement("PAGER");
        if (phone.isMsg)
            writer->writeEmptyElement("MSG");
        if (phone.isCell)
            writer->writeEmptyElement("CELL");
        if (phone.isVideo)
            writer->writeEmptyElement("VIDEO");
        if (phone.isBBS)
            writer->writeEmptyElement("BBS");
        if (phone.isModem)
            writer->writeEmptyElement("MODEM");
        if (phone.isISDN)
            writer->writeEmptyElement("ISDN");
        if (phone.isPCS)
            writer->writeEmptyElement("PCS");
        if (phone.isPref)
            writer->writeEmptyElement("PREF");
        writer->writeTextElement("NUMBER", phone.number);

        writer->writeEndElement();
    }

    foreach (const QXmppVCardEmail &email, m_emails)
    {
        writer->writeStartElement("EMAIL");

        if (email.isHome)
            writer->writeEmptyElement("HOME");
        if (email.isWork)
            writer->writeEmptyElement("WORK");
        if (email.isInternet)
            writer->writeEmptyElement("INTERNET");
        if (email.isPref)
            writer->writeEmptyElement("PREF");
        if (email.isX400)
            writer->writeEmptyElement("X400");
        writer->writeTextElement("USERID", email.address);

        writer->writeEndElement();
    }

    foreach (const QXmppVCardAddress &address, m_addresses)
    {
        writer->writeStartElement("ADR");

        if (address.isHome)
            writer->writeEmptyElement("HOME");
        if (address.isWork)
            writer->writeEmptyElement("WORK");
        if (address.isPostal)
            writer->writeEmptyElement("POSTAL");
        if (address.isPref)
            writer->writeEmptyElement("PREF");

        if (!address.country.isEmpty())
            writer->writeTextElement("CTRY", address.country);
        if (!address.extAdd.isEmpty())
            writer->writeTextElement("EXTADD", address.extAdd);
        if (!address.locality.isEmpty())
            writer->writeTextElement("LOCALITY", address.locality);
        if (!address.pCode.isEmpty())
            writer->writeTextElement("PCODE", address.pCode);
        if (!address.region.isEmpty())
            writer->writeTextElement("REGION", address.region);
        if (!address.street.isEmpty())
            writer->writeTextElement("STREET", address.street);

        writer->writeEndElement();
    }

    writer->writeEndElement();
}
/// \endcond
